package repeaterhub;

import java.util.ArrayList;
import java.util.List;
import org.opendaylight.controller.forwardingrulesmanager.FlowEntry;
import org.opendaylight.controller.forwardingrulesmanager.IForwardingRulesManager;
import org.opendaylight.controller.sal.core.Node;
import org.opendaylight.controller.sal.core.NodeConnector;
import org.opendaylight.controller.sal.core.NodeConnector.NodeConnectorIDType;
import org.opendaylight.controller.sal.match.MatchType;
import org.opendaylight.controller.sal.action.Action;
import org.opendaylight.controller.sal.action.Flood;
import org.opendaylight.controller.sal.flowprogrammer.Flow;
import org.opendaylight.controller.sal.match.Match;
import org.opendaylight.controller.sal.packet.Ethernet;
import org.opendaylight.controller.sal.packet.IDataPacketService;
import org.opendaylight.controller.sal.packet.Packet;
import org.opendaylight.controller.sal.packet.IListenDataPacket;
import org.opendaylight.controller.sal.packet.PacketResult;
import org.opendaylight.controller.sal.packet.RawPacket;
import org.opendaylight.controller.sal.utils.Status;
import org.opendaylight.controller.sal.utils.NodeConnectorCreator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class RepeaterHub implements IListenDataPacket {

    private IForwardingRulesManager frm;
    private IDataPacketService dps;
    private static final Logger logger = LoggerFactory.getLogger(RepeaterHub.class);

    void init() {
    }

    void destroy() {
    }

    void start() {
    }

    public void setForwardingRulesManager(IForwardingRulesManager forwardingRulesManager) {
        this.frm = forwardingRulesManager;
    }

    public void unsetForwardingRulesManager(IForwardingRulesManager f) {
        if (this.frm == f) {
            this.frm = null;
        }
    }

    void setDataPacketService(IDataPacketService s) {
        this.dps = s;
    }

    void unsetDataPacketService(IDataPacketService s) {
        if (this.dps == s) {
            this.dps = null;
        }
    }

    @Override
    public PacketResult receiveDataPacket(RawPacket inPkt) {
        Packet pak = dps.decodeDataPacket(inPkt);
        if (pak instanceof Ethernet) {
            // action
            List<Action> actions = new ArrayList<Action>();
            actions.add(new Flood());
            // match
            Match match = new Match();
            byte[] srcMac;
            srcMac = ((Ethernet) pak).getSourceMACAddress();
            match.setField(MatchType.DL_SRC, srcMac);
            // flow
            Flow flow = new Flow(match, actions);
            Node node = inPkt.getIncomingNodeConnector().getNode();
            String mac = convertToString(srcMac);
            FlowEntry fEntry = new FlowEntry("Policy" + mac, "Flow" + mac, flow, node);
            // install flow
            Status status = frm.installFlowEntry(fEntry);
            if (status.isSuccess()) {
                logger.trace("Flow entry was installed successfully");
            }
        }
        // packet out 
        floodPacket(inPkt);
        return PacketResult.IGNORED;
    }

    private void floodPacket(RawPacket inPkt) {
        Node node = inPkt.getIncomingNodeConnector().getNode();
        NodeConnector allNodeConnector = NodeConnectorCreator
                .createNodeConnector(NodeConnectorIDType.ALL, 
                        NodeConnector.SPECIALNODECONNECTORID, node);
        inPkt.setOutgoingNodeConnector(allNodeConnector);
        dps.transmitDataPacket(inPkt);
    }

    private String convertToString(byte[] byteArray) {
        StringBuilder sb = new StringBuilder();
        for (byte b : byteArray) {
        sb.append(String.format("%02x", b));
        }
        return sb.toString();
    }
}
